import matplotlib.pyplot as plt
from matplotlib import ticker, cm
import numpy as np
from .figfuncs import formatFigure

## Plotter functions.
def plotter(**kwargs):
    SweepType = kwargs['SweepType']

    if 'scale' in kwargs.keys():
        scale = kwargs['scale']
    else:
        scale = 'lin'

    if 'Constrain' in kwargs.keys():
        constrain = kwargs['Constrain']
    else:
        constrain = {'Bool': False}

    fig = plt.figure()
    if SweepType == '1D':

        x = kwargs['x']
        y = kwargs['y']
        xlabel = kwargs['xlabel']
        ylabel = kwargs['ylabel']
        color = kwargs['Color']

        ax = fig.add_subplot(111)

        if scale == 'lin':
            ax.plot(x, y, linewidth=2.0, color=color)
        elif scale == 'log':
            ax.semilogy(x, y, linewidth=2.0, color=color)

        if constrain['Bool']:
            if constrain['xlim_min'] != constrain['xlim_max']:
                ax.set_xlim([constrain['xlim_min'], constrain['xlim_max']])
            if constrain['ylim_min'] != constrain['ylim_max']:
                ax.set_xlim([constrain['ylim_min'], constrain['ylim_max']])

        if 'tickoffsety' in kwargs.keys():
            tickoffsety = kwargs['tickoffsety']
        else:
            tickoffsety = 0

        if 'tickseparationy' in kwargs.keys():
            tickseparationy = kwargs['tickseparationy']
        else:
            tickseparationy = 2

        if 'tickoffsetx' in kwargs.keys():
            tickoffsetx = kwargs['tickoffsetx']
        else:
            tickoffsetx = 0

        if 'tickseparationx' in kwargs.keys():
            tickseparationx = kwargs['tickseparationx']
        else:
            tickseparationx = 2

        ax = formatFigure(ax=ax, projection='2d', labels={'xlabel': xlabel, 'ylabel': ylabel},
                          tickseparationy=tickseparationy, tickoffsety=tickoffsety,
                          tickseparationx=tickseparationx, tickoffsetx=tickoffsetx)
        if 'legend' in kwargs.keys():
            if kwargs['legend']:
                ax.legend(loc='lower center', ncol=4, bbox_to_anchor=(0.5, 1.0))

        if 'show' in kwargs.keys():
            if kwargs['show']:
                plt.show()
        else:
            plt.show()

    elif SweepType == 'Multiple':
        xlist = kwargs['x']
        ylist = kwargs['y']
        xlabel = kwargs['xlabel']
        ylabel = kwargs['ylabel']
        colors = kwargs['Color']
        labels = kwargs['labels']

        ax = fig.add_subplot(111)

        for x, y, label, color in zip(xlist, ylist, labels, colors):
            if scale == 'lin':
                ax.plot(x, y, linewidth=2.0, color=color, label=label)
            if scale == 'log':
                ax.semilogy(x, abs(y), linewidth=2.0, color=color, label=label)

        if constrain['Bool']:
            if constrain['xlim_min'] != constrain['xlim_max']:
                ax.set_xlim([constrain['xlim_min'], constrain['xlim_max']])
            if constrain['ylim_min'] != constrain['ylim_max']:
                ax.set_xlim([constrain['ylim_min'], constrain['ylim_max']])

        if 'tickoffsety' in kwargs.keys():
            tickoffsety = kwargs['tickoffsety']
        else:
            tickoffsety = 0

        if 'tickseparationy' in kwargs.keys():
            tickseparationy = kwargs['tickseparationy']
        else:
            tickseparationy = 2

        if 'tickoffsetx' in kwargs.keys():
            tickoffsetx = kwargs['tickoffsetx']
        else:
            tickoffsetx = 0

        if 'tickseparationx' in kwargs.keys():
            tickseparationx = kwargs['tickseparationx']
        else:
            tickseparationx = 2

        ax = formatFigure(ax=ax, projection='2d', labels={'xlabel': xlabel, 'ylabel': ylabel},
                          tickseparationy=tickseparationy, tickoffsety=tickoffsety,
                          tickseparationx=tickseparationx, tickoffsetx=tickoffsetx)

        if 'legend' in kwargs.keys():
            if kwargs['legend']:
                ax.legend(loc='lower center', ncol=4, bbox_to_anchor=(0.5, 1.0))

        if 'show' in kwargs.keys():
            if kwargs['show']:
                plt.show()
        else:
            plt.show()

    elif SweepType == '2D':
        x = kwargs['x']
        y = kwargs['y']
        z = kwargs['z']
        xx, yy = np.meshgrid(x, y)

        xlabel = kwargs['xlabel']
        ylabel = kwargs['ylabel']
        zlabel = kwargs['zlabel']
        colorbar = kwargs['ColorBar']

        ax = fig.add_subplot(111)
        if scale == 'log':
            cp = ax.contourf(xx, yy, np.abs(z), locator=ticker.LogLocator(), cmap=colorbar)
        elif scale == 'lin':
            cp = ax.contourf(xx, yy, np.transpose(z), cmap=colorbar)

        if constrain['Bool']:
            if constrain['xlim_min'] != constrain['xlim_max']:
                ax.set_xlim([constrain['xlim_min'], constrain['xlim_max']])
            if constrain['ylim_min'] != constrain['ylim_max']:
                ax.set_xlim([constrain['ylim_min'], constrain['ylim_max']])

        cbar = plt.colorbar(cp)
        cbar.set_label(zlabel, fontsize=14)
        ax = formatFigure(ax=ax, projection='2d', labels={'xlabel': xlabel, 'ylabel': ylabel, 'zlabel': zlabel})

        if 'show' in kwargs.keys():
            if kwargs['show']:
                plt.show()
            else:
                plt.show()

    else:
        ax = 0

    return ax, fig



import numpy as np
from matplotlib import cm

def get_colors(N, cmap_name='plasma'):
    cmap = cm.get_cmap(cmap_name)
    color_id = np.linspace(0, 1, N+10)
    colors = []
    for ii in color_id:
        colors.append(cmap(ii))
    colors = colors[5:-5]
    return colors


import setuptools
with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='fGQLED',
    version='0.17',
    author='Killian Keller',
    author_email='kkeller@ethz.ch',
    description='Standard functions used for the QDLED Project at ETH.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['fgqled'],
    classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    ],
)


def formatFigure(ax, projection='2d', labels={'xlabel': '', 'ylabel': '', 'zlabel': ''}, fontsize=14, tickseparationx=2, tickseparationy=2, tickoffsetx=0, tickoffsety=0, ticksize=14):

    if projection == '2d':
        if 'xlabel' in labels.keys():
            xlabel = labels['xlabel']
            ax.set_xlabel(xlabel, fontsize=fontsize)
        if 'ylabel' in labels.keys():
            ylabel = labels['ylabel']
            ax.set_ylabel(ylabel, fontsize=fontsize)

        for ii, tick in enumerate(ax.yaxis.get_major_ticks()):
            tick.label.set_fontsize(ticksize)
            if (ii+tickoffsety) % tickseparationy != 0:
                tick.label1.set_visible(False)

        for ii, tick in enumerate(ax.xaxis.get_major_ticks()):
            tick.label.set_fontsize(ticksize)
            if (ii+tickoffsetx) % tickseparationx != 0:
                tick.label1.set_visible(False)

    elif projection == '3d':
        if 'xlabel' in labels.keys():
            xlabel = labels['xlabel']
            ax.set_xlabel(xlabel, fontsize=14, labelpad=15)
        if 'ylabel' in labels.keys():
            ylabel = labels['ylabel']
            ax.set_ylabel(ylabel, fontsize=14, labelpad=15)
        if 'zlabel' in labels.keys():
            zlabel = labels['zlabel']
            ax.set_zlabel(zlabel, fontsize=14, labelpad=15)

        for ii, tick in enumerate(ax.yaxis.get_major_ticks()):
            tick.label.set_fontsize(ticksize)
            if (ii+tickoffsety) % tickseparationy != 0:
                tick.label1.set_visible(False)

        for ii, tick in enumerate(ax.xaxis.get_major_ticks()):
            tick.label.set_fontsize(ticksize)
            if (ii+tickoffsetx) % tickseparationx != 0:
                tick.label1.set_visible(False)

        for ii, tick in enumerate(ax.zaxis.get_major_ticks()):
            tick.label.set_fontsize(ticksize)
            if ii % 2 != 0:
                tick.label1.set_visible(False)

        ax.view_init(20,-120)


    return ax



import numpy as np

def reader(**kwargs):
    assert ('filename' in kwargs.keys()), 'Filename cannot be empty'
    filename = kwargs['filename']

    if not 'allowedChars' in kwargs.keys():
        allowedChars = ['%d' % ii for ii in range(10)]
        allowedChars.extend(['-', '+'])
    else:
        allowedChars = kwargs['allowedChars']

    if not 'datatype' in kwargs.keys():
        datatype = float
    else:
        datatype = kwargs['datatype']

    if not 'delimiter' in kwargs.keys():
        delimiter = ' '
    else:
        delimiter = kwargs['delimiter']

    if not 'skipLines' in kwargs.keys():
        skipLines = 0
    else:
        skipLines = kwargs['skipLines']

    if not 'flatten' in kwargs.keys():
        flatten = 0
    else:
        flatten = kwargs['flatten']

    f = open(filename, 'r')
    data = []
    for kk, l in enumerate(f):
        if kk < skipLines:
            continue
        if l[0] not in allowedChars:
            continue

        tempChars = []
        tempStr = []
        for c in l:
            # Once a newline is reached, saves the vector into the next line of the data Matrix.
            if c == '\n':
                tempStr.append(''.join(tempChars))
                data.append(tempStr)
                break
            # Once a delimiter is hit, the characterArray is changed into a string and the string is put into the vector
            if c == delimiter:
                if tempChars != []:
                    tempStr.append(''.join(tempChars))
                    tempChars = []
                continue
            # reads until hits a delimiter
            elif c != delimiter:
                tempChars.append(c)
                continue

    for ii, d in enumerate(data):
        if d[-1] == '':
            data[ii] = d[:-1]

    if not data[-1]:
        data = data[:-1]

    if not flatten == 0:
        temp = []
        for ii, d in enumerate(data):
            if len(d) == flatten:
                temp.append(d)
        data = temp

    if datatype == complex:
        for ii, l in enumerate(data):
            for kk, d in enumerate(l):
                if '+' in d:
                    realPart = float(d[:d.index('+')])
                    imagPart = float(d[d.index('+'):d.index('i')])
                    l[kk] = realPart + 1j*imagPart
                if '-' in d:
                    realPart = float(d[:d.index('-')])
                    imagPart = float(d[d.index('-')+len('-'):d.index('i')])
                    l[kk] = realPart - 1j*imagPart
            data[ii] = l

    data = np.asarray(data, dtype=datatype)

    return data
